@@ -90,6 +90,7 @@
 #include "nsIPrompt.h"
 #include "nsISupportsArray.h"
 #include "nsIMsgSearchTerm.h"
+#include "nsIAtomService.h"
 
 static NS_DEFINE_CID(kImapUrlCID, NS_IMAPURL_CID);
 static NS_DEFINE_CID(kCMailboxUrl, NS_MAILBOXURL_CID);
@@ -99,6 +100,8 @@ static NS_DEFINE_CID(kCNntpUrlCID, NS_NNTPURL_CID);
 #define ILLEGAL_FOLDER_CHARS_AS_FIRST_LETTER "."
 #define ILLEGAL_FOLDER_CHARS_AS_LAST_LETTER  ".~ "
 
+#define FOUR_K 4096
+
 nsresult GetMessageServiceContractIDForURI(const char *uri, nsCString &contractID)
 {
   nsresult rv = NS_OK;
@@ -318,9 +321,12 @@ inline PRUint32 StringHash(const nsAutoString& str)
 
 /* Utility functions used in a few places in mailnews */
 PRInt32
-FindCharInSet(const nsCString &aString,
-              const char* aChars, PRUint32 aOffset) 
+MsgFindCharInSet(const nsCString &aString,
+                 const char* aChars, PRUint32 aOffset)
 {
+#ifdef MOZILLA_INTENAL_API
+  return FindCharInSet(aChars, aOffset)
+#else
   PRInt32 len = strlen(aChars);
   PRInt32 index = -1;
   for (int i = aOffset; i < len; i++) {
@@ -329,12 +335,16 @@ FindCharInSet(const nsCString &aString,
       return index;
   }
   return -1;
+#endif
 }
 
 PRInt32
-FindCharInSet(const nsString &aString,
-              const char* aChars, PRUint32 aOffset)
+MsgFindCharInSet(const nsString &aString,
+                 const char* aChars, PRUint32 aOffset)
 {
+#ifdef MOZILLA_INTENAL_API
+  return FindCharInSet(aChars, aOffset)
+#else
   PRInt32 len = strlen(aChars);
   PRInt32 index = -1;
   for (int i = aOffset; i < len; i++) {
@@ -343,32 +353,7 @@ FindCharInSet(const nsString &aString,
       return index;
   }
   return -1;
-}
-
-PRInt32
-RFindCharInSet(const nsCString &aString, const char* aChars) 
-{
-  PRInt32 len = strlen(aChars);
-  PRInt32 index = -1;
-  for (int i = 0; i < len; i++) {
-    index = aString.RFindChar(aChars[i]);
-    if (index != -1)
-      return index;
-  }
-  return -1;
-}
-
-PRInt32
-RFindCharInSet(const nsString &aString, const char* aChars) 
-{
-  PRInt32 len = strlen(aChars);
-  PRInt32 index = -1;
-  for (int i = 0; i < len; i++) {
-    index = aString.RFindChar(aChars[i]);
-    if (index != -1)
-      return index;
-  }
-  return -1;
+#endif
 }
 
 // XXX : this may have other clients, in which case we'd better move it to
@@ -399,10 +384,10 @@ nsresult NS_MsgHashIfNecessary(nsCAutoString &name)
   // Given a filename, make it safe for filesystem
   // certain filenames require hashing because they
   // are too long or contain illegal characters
-  PRInt32 illegalCharacterIndex = FindCharInSet(str, 
-                                                FILE_PATH_SEPARATOR 
-                                                FILE_ILLEGAL_CHARACTERS 
-                                                ILLEGAL_FOLDER_CHARS);
+  PRInt32 illegalCharacterIndex = MsgFindCharInSet(str,
+                                                   FILE_PATH_SEPARATOR
+                                                   FILE_ILLEGAL_CHARACTERS
+                                                   ILLEGAL_FOLDER_CHARS, 0);
 
   // Need to check the first ('.') and last ('.', '~' and ' ') char
   if (illegalCharacterIndex == -1)
@@ -452,10 +437,10 @@ nsresult NS_MsgHashIfNecessary(nsCAutoString &name)
 // because MAX_LEN is defined rather conservatively in the first place.
 nsresult NS_MsgHashIfNecessary(nsAutoString &name)
 {
-  PRInt32 illegalCharacterIndex = FindCharInSet(name,
-                                                FILE_PATH_SEPARATOR 
-                                                FILE_ILLEGAL_CHARACTERS 
-                                                ILLEGAL_FOLDER_CHARS);
+  PRInt32 illegalCharacterIndex = MsgFindCharInSet(name,
+                                                   FILE_PATH_SEPARATOR
+                                                   FILE_ILLEGAL_CHARACTERS
+                                                   ILLEGAL_FOLDER_CHARS, 0);
 
   // Need to check the first ('.') and last ('.', '~' and ' ') char
   if (illegalCharacterIndex == -1)
@@ -511,7 +496,8 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *aFolderURI,
   if (endSlashPos < 0)
     endSlashPos = oldPath.Length();
 #ifdef XP_MACOSX
-  PRBool isMailboxUri = aScheme.EqualsLiteral("none");
+  PRBool isMailboxUri = aScheme.EqualsLiteral("none") ||
+                        aScheme.EqualsLiteral("pop3");
 #endif
   // trick to make sure we only add the path to the first n-1 folders
   PRBool haveFirst=PR_FALSE;
@@ -554,7 +540,13 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *aFolderURI,
     if (startSlashPos >= endSlashPos)
       break;
   }
+#ifdef MOZILLA_INTERNAL_API
   return NS_CopyUnicodeToNative(path, aPathCString);
+#else
+  NS_ERROR("NS_CopyUnicodeToNative not implemented in frozen linkage.");
+  LossyCopyUTF16toASCII(path, aPathCString);
+  return NS_OK;
+#endif
 }
 
 PRBool NS_MsgStripRE(const char **stringP, PRUint32 *lengthP, char **modifiedSubject)
@@ -779,8 +771,12 @@ nsresult GetExistingFolder(const nsCString& aFolderURI, nsIMsgFolder **aFolder)
   // Parent doesn't exist means that this folder doesn't exist.
   nsCOMPtr<nsIMsgFolder> parentFolder;
   rv = thisFolder->GetParent(getter_AddRefs(parentFolder));
-  if (NS_SUCCEEDED(rv) && parentFolder)
+  if (NS_SUCCEEDED(rv)) {
+    // When parentFolder is null with NS_OK, we should return error.
+    NS_ENSURE_TRUE(parentFolder, NS_ERROR_FAILURE);
+
     NS_ADDREF(*aFolder = thisFolder);
+  }
   return rv;
 }
 
@@ -809,9 +805,10 @@ nsresult EscapeFromSpaceLine(nsIOutputStream *outputStream, char *start, const c
   pChar = start;
   while (start < end)
   {
-    while ((pChar < end) && (*pChar != '\r') && ((pChar+1) < end) && (*(pChar+1) != '\n'))
+    while ((pChar < end) && (*pChar != '\r') && ((pChar + 1) < end) &&
+           (*(pChar + 1) != '\n'))
       pChar++;
-    if ((pChar+1) == end)
+    if ((pChar + 1) == end)
       pChar++;
 
     if (pChar < end)
@@ -1337,6 +1334,18 @@ nsresult MsgReopenFileStream(nsILocalFile *file, nsIInputStream *fileStream)
     return NS_ERROR_FAILURE;
 }
 
+nsresult MsgNewBufferedFileOutputStream(nsIOutputStream **aResult,
+                                        nsIFile* aFile,
+                                        PRInt32 aIOFlags,
+                                        PRInt32 aPerm)
+{
+  nsCOMPtr<nsIOutputStream> stream;
+  nsresult rv = NS_NewLocalFileOutputStream(getter_AddRefs(stream), aFile, aIOFlags, aPerm);
+  if (NS_SUCCEEDED(rv))
+    rv = NS_NewBufferedOutputStream(aResult, stream, FOUR_K);
+  return rv;
+}
+
 PRBool MsgFindKeyword(const nsCString &keyword, nsCString &keywords, PRInt32 *aStartOfKeyword, PRInt32 *aLength)
 {
 #ifdef MOZILLA_INTERNAL_API
@@ -1421,42 +1430,6 @@ PRBool MsgHostDomainIsTrusted(nsCString &host, nsCString &trustedMailDomains)
   return domainIsTrusted;
 }
 
-nsresult FolderUriFromDirInProfile(nsILocalFile *aLocalPath, nsACString &mailboxUri)
-{
-  nsresult rv;
-
-  nsCOMPtr<nsIMsgAccountManager> accountManager =
-    do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIArray> folderArray;
-  rv = accountManager->GetAllFolders(getter_AddRefs(folderArray));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  PRUint32 count;
-  rv = folderArray->GetLength(&count);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  for (PRUint32 i = 0; i < count; i++)
-  {
-    nsCOMPtr<nsIMsgFolder> folder(do_QueryElementAt(folderArray, i, &rv));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsILocalFile> folderPath;
-    rv = folder->GetFilePath(getter_AddRefs(folderPath));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    // Check if we're equal
-    PRBool isEqual;
-    rv = folderPath->Equals(aLocalPath, &isEqual);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    if (isEqual)
-      return folder->GetURI(mailboxUri);
-  }
-  return NS_ERROR_FAILURE;
-}
-
 nsresult MsgGetLocalFileFromURI(const nsACString &aUTF8Path, nsILocalFile **aFile)
 {
   nsresult rv;
@@ -1472,6 +1445,7 @@ nsresult MsgGetLocalFileFromURI(const nsACString &aUTF8Path, nsILocalFile **aFil
   return CallQueryInterface(argFile, aFile);
 }
 
+#ifndef MOZILLA_INTERNAL_API
 /*
  * Function copied from nsReadableUtils.
  * Migrating to frozen linkage is the only change done
@@ -1548,6 +1522,8 @@ NS_MSG_BASE PRBool MsgIsUTF8(const nsACString& aString)
   return !state; // state != 0 at the end indicates an invalid UTF-8 seq. 
 }
 
+#endif
+
 NS_MSG_BASE void MsgStripQuotedPrintable (unsigned char *src)
 {
   // decode quoted printable text in place
@@ -1651,6 +1627,8 @@ NS_MSG_BASE nsresult MsgEscapeURL(const nsACString &aStr, PRUint32 aFlags,
   return nu->EscapeURL(aStr, aFlags, aResult);
 }
 
+#ifndef MOZILLA_INTERNAL_API
+
 NS_MSG_BASE char *MsgEscapeHTML(const char *string)
 {
   char *rv = nsnull;
@@ -1715,7 +1693,7 @@ NS_MSG_BASE char *MsgEscapeHTML(const char *string)
   return(rv);
 }
 
-NS_MSG_BASE PRUnichar *MsgEscapeHTML(const PRUnichar *aSourceBuffer, 
+NS_MSG_BASE PRUnichar *MsgEscapeHTML2(const PRUnichar *aSourceBuffer,
                                       PRInt32 aSourceBufferLen)
 {
   // if the caller didn't calculate the length
@@ -1778,10 +1756,6 @@ NS_MSG_BASE PRUnichar *MsgEscapeHTML(const PRUnichar *aSourceBuffer,
 
 NS_MSG_BASE void MsgCompressWhitespace(nsCString& aString)
 {
-#ifdef MOZILLA_INTERNAL_API
-  // Use the convenience function provided by the internal API.
-  aString.CompressWhitespace(PR_TRUE, PR_TRUE);
-#else
   // This code is frozen linkage specific
   aString.Trim(" \f\n\r\t\v");
 
@@ -1815,9 +1789,123 @@ NS_MSG_BASE void MsgCompressWhitespace(nsCString& aString)
 
   // Set the new length.
   aString.SetLength(end - start);
-#endif
 }
 
+NS_MSG_BASE void MsgReplaceChar(nsString& str, const char *set, const PRUnichar replacement)
+{
+  PRUnichar *c_str = str.BeginWriting();
+  while (*set) {
+    PRInt32 pos = 0;
+    while ((pos = str.FindChar(*set, pos)) != -1) {
+      c_str[pos++] = replacement;
+    }
+    set++;
+  }
+}
+
+NS_MSG_BASE void MsgReplaceChar(nsCString& str, const char needle, const char replacement)
+{
+  char *c_str = str.BeginWriting();
+  while ((c_str = strchr(c_str, needle))) {
+    *c_str = replacement;
+    c_str++;
+  }
+}
+
+NS_MSG_BASE nsIAtom* MsgNewAtom(const char* aString)
+{
+  nsCOMPtr<nsIAtomService> atomService(do_GetService("@mozilla.org/atom-service;1"));
+  nsIAtom* atom = nsnull;
+
+  if (atomService)
+    atomService->GetAtomUTF8(aString, &atom);
+  return atom;
+}
+
+NS_MSG_BASE void MsgReplaceSubstring(nsAString &str, const nsAString &what, const nsAString &replacement)
+{
+  const PRUnichar* replacement_str;
+  PRUint32 replacementLength = replacement.BeginReading(&replacement_str);
+  PRUint32 whatLength = what.Length();
+  PRInt32 i = 0;
+
+  while ((i = str.Find(what, i)) != kNotFound)
+  {
+    str.Replace(i, whatLength, replacement_str, replacementLength);
+    i += replacementLength;
+  }
+}
+
+NS_MSG_BASE void MsgReplaceSubstring(nsACString &str, const char *what, const char *replacement)
+{
+  PRUint32 replacementLength = strlen(replacement);
+  PRUint32 whatLength = strlen(what);
+  PRInt32 i = 0;
+
+  while ((i = str.Find(what, i)) != kNotFound)
+  {
+    str.Replace(i, whatLength, replacement, replacementLength);
+    i += replacementLength;
+  }
+}
+
+/* This class is based on nsInterfaceRequestorAgg from nsInterfaceRequestorAgg.h */
+class MsgInterfaceRequestorAgg : public nsIInterfaceRequestor
+{
+public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIINTERFACEREQUESTOR
+
+  MsgInterfaceRequestorAgg(nsIInterfaceRequestor *aFirst,
+                           nsIInterfaceRequestor *aSecond)
+    : mFirst(aFirst)
+    , mSecond(aSecond) {}
+
+  nsCOMPtr<nsIInterfaceRequestor> mFirst, mSecond;
+};
+
+// XXX This needs to support threadsafe refcounting until we fix bug 243591.
+NS_IMPL_THREADSAFE_ISUPPORTS1(MsgInterfaceRequestorAgg, nsIInterfaceRequestor)
+
+NS_IMETHODIMP
+MsgInterfaceRequestorAgg::GetInterface(const nsIID &aIID, void **aResult)
+{
+  nsresult rv = NS_ERROR_NO_INTERFACE;
+  if (mFirst)
+    rv = mFirst->GetInterface(aIID, aResult);
+  if (mSecond && NS_FAILED(rv))
+    rv = mSecond->GetInterface(aIID, aResult);
+  return rv;
+}
+
+/* This function is based on NS_NewInterfaceRequestorAggregation from
+ * nsInterfaceRequestorAgg.h */
+NS_MSG_BASE nsresult
+MsgNewInterfaceRequestorAggregation(nsIInterfaceRequestor *aFirst,
+                                    nsIInterfaceRequestor *aSecond,
+                                    nsIInterfaceRequestor **aResult)
+{
+  *aResult = new MsgInterfaceRequestorAgg(aFirst, aSecond);
+  if (!*aResult)
+    return NS_ERROR_OUT_OF_MEMORY;
+  NS_ADDREF(*aResult);
+  return NS_OK;
+}
+
+nsresult NS_FASTCALL MsgQueryElementAt::operator()( const nsIID& aIID, void** aResult ) const
+  {
+    nsresult status = mArray
+                        ? mArray->QueryElementAt(mIndex, aIID, aResult)
+                        : NS_ERROR_NULL_POINTER;
+
+    if ( mErrorPtr )
+      *mErrorPtr = status;
+
+    return status;
+  }
+
+#endif
+
 NS_MSG_BASE nsresult MsgGetHeadersFromKeys(nsIMsgDatabase *aDB, const nsTArray<nsMsgKey> &aMsgKeys,
                                            nsIMutableArray *aHeaders)
 {
@@ -2009,3 +2097,15 @@ NS_MSG_BASE nsresult MsgTermListToString(nsISupportsArray *aTermList, nsCString
   }
   return rv;
 }
+
+NS_MSG_BASE PRUint64 ParseUint64Str(const char *str)
+{
+#ifdef XP_WIN
+  {
+    char *endPtr;
+    return _strtoui64(str, &endPtr, 10);
+  }
+#else
+  return strtoull(str, nsnull, 10);
+#endif
+}
